<?php

namespace Drupal\recurly_aegir\HostingServiceCalls;

use Drupal\node\Entity\Node;

/**
 * Sets a quota for a site.
 */
class SiteQuotaHostingServiceCall extends SiteHostingServiceCall {

  /**
   * The activity that was performed by this hosting service call's execution.
   */
  const ACTION_PERFORMED = 'Site quota set';

  /**
   * The ID of the quota to set.
   *
   * @var string
   */
  protected $quota;

  /**
   * The limit of the quota to set.
   *
   * @var int
   */
  protected $limit;

  /**
   * Class Constructor.
   *
   * @param Drupal\node\Entity\Node $site
   *   The name of the site whose quotas are to be set.
   * @param string $quota
   *   The ID of the quota to set.
   * @param string $limit
   *   The limit of the quota to set.
   */
  public function __construct(Node $site, $quota, $limit) {
    parent::__construct($site);

    if (!is_array($quota)) {
      throw new \Exception('Quota hosting service calls must be provided with a quota ID on construction.');
    }
    if (!is_array($limit)) {
      throw new \Exception('Quota hosting service calls must be provided with a quota limit on construction.');
    }

    $this->quota = $quota;
    $this->limit = $limit;
  }

  /**
   * {@inheritdoc}
   *
   * Set a site's quota.
   */
  protected function execute() {
    $this->sendRequestAndReceiveResponse('variables', [
      'site' => $this->getSiteName(),
      'name' => $this->quota,
      'value' => $this->limit,
    ]);

    return $this;
  }

  /**
   * Set quotas for a particular site based on its subscription plan & add-ons.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site whose quota limits are to be set.
   * @param string $plan_code
   *   The plan code for this site's subscription.
   * @param array $addons
   *   The list of add-ons included in the subscription.
   *
   * @see https://www.drupal.org/project/quenforcer
   */
  public static function setQuotas(Node $site, $plan_code, array $addons) {
    $quotas = \Drupal::moduleHandler()->invokeAll('recurly_aegir_quota_info', [
      $site,
      $plan_code,
      $addons,
    ]);

    foreach ($quotas as $quota => $limit) {
      $quota_setter = new static($site, $quota, $limit);
      $quota_setter->performActionAndLogResults();
    }

    if (!empty($quotas)) {
      $site_verifier = new SiteVerifyHostingServiceCall($site);
      $site_verifier->performActionAndLogResults();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function recordSuccessLogMessage() {
    $this->getLogger()
      ->info('Remote site %sitename: The limit for quota %quota was set to %limit via %class.', [
        '%sitename' => $this->getSiteName(),
        '%quota' => $this->quota,
        '%limit' => $this->limit,
        '%class' => $this->getClassName(),
      ]);
    return $this;
  }

}
