<?php

namespace Drupal\recurly_aegir\WebhookNotificationHandlers;

use Drupal\recurly_aegir\Wrappers\SubscriptionWrapper;

/**
 * Processes Recurly notifications for Aegir.
 */
abstract class WebhookNotificationHandler {

  const NOTIFICATION_MAP = [
    'new_account_notification' => 'NewAccountWebhookNotificationHandler',
    'canceled_account_notification' => 'CanceledAccountWebhookNotificationHandler',
    'billing_info_updated_notification' => 'UpdatedAccountWebhookNotificationHandler',
    'reactivated_account_notification' => 'ReactivatedAccountWebhookNotificationHandler',
    'new_invoice_notification' => 'NewInvoiceWebhookNotificationHandler',
    'processing_invoice_notification' => 'ProcessingInvoiceWebhookNotificationHandler',
    'closed_invoice_notification' => 'ClosedInvoiceWebhookNotificationHandler',
    'past_due_invoice_notification' => 'OverdueInvoiceWebhookNotificationHandler',
    'new_subscription_notification' => 'NewSubscriptionWebhookNotificationHandler',
    'updated_subscription_notification' => 'UpdatedSubscriptionWebhookNotificationHandler',
    'canceled_subscription_notification' => 'CanceledSubscriptionWebhookNotificationHandler',
    'expired_subscription_notification' => 'ExpiredSubscriptionWebhookNotificationHandler',
    'renewed_subscription_notification' => 'RenewedSubscriptionWebhookNotificationHandler',
    'scheduled_payment_notification' => 'ScheduledPaymentWebhookNotificationHandler',
    'processing_payment_notification' => 'ProcessingPaymentWebhookNotificationHandler',
    'successful_payment_notification' => 'SuccessfulPaymentWebhookNotificationHandler',
    'failed_payment_notification' => 'FailedPaymentWebhookNotificationHandler',
    'successful_refund_notification' => 'SuccessfulRefundPaymentWebhookNotificationHandler',
    'void_payment_notification' => 'VoidPaymentWebhookNotificationHandler',
  ];

  /**
   * Account information.
   *
   * @var SimpleXMLElement|null
   */
  protected $account;

  /**
   * Subscription information.
   *
   * @var Drupal\recurly_aegir\Wrappers\SubscriptionWrapper|null
   */
  protected $subscription;

  /**
   * Transaction information.
   *
   * @var SimpleXMLElement|null
   */
  protected $transaction;

  /**
   * Invoice information.
   *
   * @var SimpleXMLElement|null
   */
  protected $invoice;

  protected $result = FALSE;

  /**
   * Class constuctor.
   *
   * @param SimpleXMLElement|null $account
   *   Account information.
   * @param SimpleXMLElement|null $subscription
   *   Subscription information.
   * @param SimpleXMLElement|null $transaction
   *   Transaction information.
   * @param SimpleXMLElement|null $invoice
   *   Invoice information.
   */
  public function __construct(
      \SimpleXMLElement $account = NULL,
      \SimpleXMLElement $subscription = NULL,
      \SimpleXMLElement $transaction = NULL,
      \SimpleXMLElement $invoice = NULL) {
    $this->account = $account;
    $this->subscription = !empty($subscription->uuid) ? SubscriptionWrapper::get((string) $subscription->uuid) : NULL;
    $this->transaction = $transaction;
    $this->invoice = $invoice;
  }

  /**
   * Handle a webhook notification from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function handleNotification();

  /**
   * Get the processing result.
   *
   * @return bool
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Returns the logging service object.
   */
  public static function getLogger() {
    return \Drupal::logger('recurly_aegir');
  }

  /**
   * Fetch the remote account code.
   *
   * @return string
   *   The remote account code.
   */
  protected function getAccountCode() {
    return (string) $this->account->account_code;
  }

  /**
   * Log a warning message and return the current object.
   */
  protected function failAndLogMessage($message) {
    $this->getLogger()->warning($message, [
      '%uuid' => $this->subscription->getUuid(),
      '%user' => $this->subscription->getLocalUserId($this->getAccountCode()),
    ]);

    $this->result = FALSE;
    return $this;
  }

}
