<?php

namespace Drupal\recurly_aegir\Wrappers;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;

/**
 * Wrapper for site nodes providing additional functionality.
 */
class SiteWrapper extends Wrapper {

  /**
   * The site node.
   *
   * @var Drupal\node\Entity\Node
   */
  protected $site;

  /**
   * Class Constructor.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site node.
   */
  public function __construct(Node $site) {
    if ($site->getType() != 'recurly_aegir_site') {
      throw new \Exception('A site node must be provided on construction.');
    }

    $this->site = $site;
  }

  /**
   * Fetches the site's subscription ID.
   *
   * @return string
   *   The subscription ID.
   */
  public function getSubscriptionId() {
    return $this->site->get('field_site_subscription_url')->getValue()[0]['title'];
  }

  /**
   * Fetches the subscription wrapper associated with the site.
   *
   * @return Drupal\recurly_aegir\Wrappers\SubscriptionWrapper
   *   The subscription wrapper.
   */
  public function getSubscription() {
    return SubscriptionWrapper::get($this->getSubscriptionId());
  }

  /**
   * Determines if a particular subscription is active for a site.
   *
   * @return bool
   *   TRUE if subscription is active; FALSE otherwise.
   */
  public function subscriptionIsActive() {
    if (!recurly_client_initialize()) {
      return FALSE;
    }

    return SubscriptionWrapper::get($this->getSubscriptionId())->isActive();
  }

  /**
   * Gets the configuration link for a specific site.
   *
   * @return Drupal\Core\Link
   *   The site node's configuration link.
   */
  public function getConfigurationLink() {
    $url = Url::fromRoute('entity.node.edit_form', ['node' => $this->site->id()]);
    return Link::fromTextAndUrl(t('<em>Configuration required!</em>'), $url);
  }

  /**
   * Fetches the site's link.
   *
   * @return Drupal\Core\Link
   *   A link to the site itself.
   */
  public function getLink() {
    $protocol = \Drupal::request()->getScheme();
    $subdomain = Html::escape($this->getTitle());
    $domain = \Drupal::request()->getHost();
    $hostname = "$subdomain.$domain";
    $url = Url::fromUri("$protocol://$hostname/");
    return Link::fromTextAndUrl($hostname, $url);
  }

  /**
   * Fetches the site's title.
   *
   * @return string
   *   The site's title.
   */
  public function getTitle() {
    return $this->site->getTitle();
  }

  /**
   * Saves the site node.
   */
  public function save() {
    return $this->site->save();
  }

}
